// Code generated by command: go run valid_asm.go -pkg ascii -out ../ascii/valid_amd64.s -stubs ../ascii/valid_amd64.go. DO NOT EDIT.

//go:build !purego

package ascii

// ValidString returns true if s contains only ASCII characters.
func ValidString(s string) bool
