// Copyright 2020-2026 Buf Technologies, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package bufmodule

import "io"

// File is a file within a Module.
type File interface {
	FileInfo
	io.ReadCloser

	isFile()
}

// *** PRIVATE ***

type file struct {
	FileInfo
	io.ReadCloser
}

func newFile(fileInfo FileInfo, readCloser io.ReadCloser) *file {
	return &file{
		FileInfo:   fileInfo,
		ReadCloser: readCloser,
	}
}

func (*file) isFile() {}
