// Copyright 2024-2025 Buf Technologies, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        (unknown)
// source: buf/plugin/info/v1/plugin_info_service.proto

//go:build !protoopaque

package infov1

import (
	_ "buf.build/gen/go/bufbuild/protovalidate/protocolbuffers/go/buf/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// A request to get information about a plugin.
type GetPluginInfoRequest struct {
	state         protoimpl.MessageState `protogen:"hybrid.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetPluginInfoRequest) Reset() {
	*x = GetPluginInfoRequest{}
	mi := &file_buf_plugin_info_v1_plugin_info_service_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetPluginInfoRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetPluginInfoRequest) ProtoMessage() {}

func (x *GetPluginInfoRequest) ProtoReflect() protoreflect.Message {
	mi := &file_buf_plugin_info_v1_plugin_info_service_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

type GetPluginInfoRequest_builder struct {
	_ [0]func() // Prevents comparability and use of unkeyed literals for the builder.

}

func (b0 GetPluginInfoRequest_builder) Build() *GetPluginInfoRequest {
	m0 := &GetPluginInfoRequest{}
	b, x := &b0, m0
	_, _ = b, x
	return m0
}

// A response containing information about a plugin.
type GetPluginInfoResponse struct {
	state protoimpl.MessageState `protogen:"hybrid.v1"`
	// The information about the plugin.
	PluginInfo    *PluginInfo `protobuf:"bytes,1,opt,name=plugin_info,json=pluginInfo,proto3" json:"plugin_info,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetPluginInfoResponse) Reset() {
	*x = GetPluginInfoResponse{}
	mi := &file_buf_plugin_info_v1_plugin_info_service_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetPluginInfoResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetPluginInfoResponse) ProtoMessage() {}

func (x *GetPluginInfoResponse) ProtoReflect() protoreflect.Message {
	mi := &file_buf_plugin_info_v1_plugin_info_service_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

func (x *GetPluginInfoResponse) GetPluginInfo() *PluginInfo {
	if x != nil {
		return x.PluginInfo
	}
	return nil
}

func (x *GetPluginInfoResponse) SetPluginInfo(v *PluginInfo) {
	x.PluginInfo = v
}

func (x *GetPluginInfoResponse) HasPluginInfo() bool {
	if x == nil {
		return false
	}
	return x.PluginInfo != nil
}

func (x *GetPluginInfoResponse) ClearPluginInfo() {
	x.PluginInfo = nil
}

type GetPluginInfoResponse_builder struct {
	_ [0]func() // Prevents comparability and use of unkeyed literals for the builder.

	// The information about the plugin.
	PluginInfo *PluginInfo
}

func (b0 GetPluginInfoResponse_builder) Build() *GetPluginInfoResponse {
	m0 := &GetPluginInfoResponse{}
	b, x := &b0, m0
	_, _ = b, x
	x.PluginInfo = b.PluginInfo
	return m0
}

var File_buf_plugin_info_v1_plugin_info_service_proto protoreflect.FileDescriptor

const file_buf_plugin_info_v1_plugin_info_service_proto_rawDesc = "" +
	"\n" +
	",buf/plugin/info/v1/plugin_info_service.proto\x12\x12buf.plugin.info.v1\x1a$buf/plugin/info/v1/plugin_info.proto\x1a\x1bbuf/validate/validate.proto\"\x16\n" +
	"\x14GetPluginInfoRequest\"`\n" +
	"\x15GetPluginInfoResponse\x12G\n" +
	"\vplugin_info\x18\x01 \x01(\v2\x1e.buf.plugin.info.v1.PluginInfoB\x06\xbaH\x03\xc8\x01\x01R\n" +
	"pluginInfo2~\n" +
	"\x11PluginInfoService\x12i\n" +
	"\rGetPluginInfo\x12(.buf.plugin.info.v1.GetPluginInfoRequest\x1a).buf.plugin.info.v1.GetPluginInfoResponse\"\x03\x90\x02\x01BRZPbuf.build/gen/go/bufbuild/bufplugin/protocolbuffers/go/buf/plugin/info/v1;infov1b\x06proto3"

var file_buf_plugin_info_v1_plugin_info_service_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_buf_plugin_info_v1_plugin_info_service_proto_goTypes = []any{
	(*GetPluginInfoRequest)(nil),  // 0: buf.plugin.info.v1.GetPluginInfoRequest
	(*GetPluginInfoResponse)(nil), // 1: buf.plugin.info.v1.GetPluginInfoResponse
	(*PluginInfo)(nil),            // 2: buf.plugin.info.v1.PluginInfo
}
var file_buf_plugin_info_v1_plugin_info_service_proto_depIdxs = []int32{
	2, // 0: buf.plugin.info.v1.GetPluginInfoResponse.plugin_info:type_name -> buf.plugin.info.v1.PluginInfo
	0, // 1: buf.plugin.info.v1.PluginInfoService.GetPluginInfo:input_type -> buf.plugin.info.v1.GetPluginInfoRequest
	1, // 2: buf.plugin.info.v1.PluginInfoService.GetPluginInfo:output_type -> buf.plugin.info.v1.GetPluginInfoResponse
	2, // [2:3] is the sub-list for method output_type
	1, // [1:2] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_buf_plugin_info_v1_plugin_info_service_proto_init() }
func file_buf_plugin_info_v1_plugin_info_service_proto_init() {
	if File_buf_plugin_info_v1_plugin_info_service_proto != nil {
		return
	}
	file_buf_plugin_info_v1_plugin_info_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_buf_plugin_info_v1_plugin_info_service_proto_rawDesc), len(file_buf_plugin_info_v1_plugin_info_service_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_buf_plugin_info_v1_plugin_info_service_proto_goTypes,
		DependencyIndexes: file_buf_plugin_info_v1_plugin_info_service_proto_depIdxs,
		MessageInfos:      file_buf_plugin_info_v1_plugin_info_service_proto_msgTypes,
	}.Build()
	File_buf_plugin_info_v1_plugin_info_service_proto = out.File
	file_buf_plugin_info_v1_plugin_info_service_proto_goTypes = nil
	file_buf_plugin_info_v1_plugin_info_service_proto_depIdxs = nil
}
