// Copyright 2023-2025 Buf Technologies, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        (unknown)
// source: buf/registry/plugin/v1beta1/compression.proto

//go:build !protoopaque

package pluginv1beta1

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The type of compression.
type CompressionType int32

const (
	CompressionType_COMPRESSION_TYPE_UNSPECIFIED CompressionType = 0
	// No compression.
	CompressionType_COMPRESSION_TYPE_NONE CompressionType = 1
	// Zstandard compression.
	CompressionType_COMPRESSION_TYPE_ZSTD CompressionType = 2
)

// Enum value maps for CompressionType.
var (
	CompressionType_name = map[int32]string{
		0: "COMPRESSION_TYPE_UNSPECIFIED",
		1: "COMPRESSION_TYPE_NONE",
		2: "COMPRESSION_TYPE_ZSTD",
	}
	CompressionType_value = map[string]int32{
		"COMPRESSION_TYPE_UNSPECIFIED": 0,
		"COMPRESSION_TYPE_NONE":        1,
		"COMPRESSION_TYPE_ZSTD":        2,
	}
)

func (x CompressionType) Enum() *CompressionType {
	p := new(CompressionType)
	*p = x
	return p
}

func (x CompressionType) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (CompressionType) Descriptor() protoreflect.EnumDescriptor {
	return file_buf_registry_plugin_v1beta1_compression_proto_enumTypes[0].Descriptor()
}

func (CompressionType) Type() protoreflect.EnumType {
	return &file_buf_registry_plugin_v1beta1_compression_proto_enumTypes[0]
}

func (x CompressionType) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

var File_buf_registry_plugin_v1beta1_compression_proto protoreflect.FileDescriptor

const file_buf_registry_plugin_v1beta1_compression_proto_rawDesc = "" +
	"\n" +
	"-buf/registry/plugin/v1beta1/compression.proto\x12\x1bbuf.registry.plugin.v1beta1*i\n" +
	"\x0fCompressionType\x12 \n" +
	"\x1cCOMPRESSION_TYPE_UNSPECIFIED\x10\x00\x12\x19\n" +
	"\x15COMPRESSION_TYPE_NONE\x10\x01\x12\x19\n" +
	"\x15COMPRESSION_TYPE_ZSTD\x10\x02BaZ_buf.build/gen/go/bufbuild/registry/protocolbuffers/go/buf/registry/plugin/v1beta1;pluginv1beta1b\x06proto3"

var file_buf_registry_plugin_v1beta1_compression_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_buf_registry_plugin_v1beta1_compression_proto_goTypes = []any{
	(CompressionType)(0), // 0: buf.registry.plugin.v1beta1.CompressionType
}
var file_buf_registry_plugin_v1beta1_compression_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_buf_registry_plugin_v1beta1_compression_proto_init() }
func file_buf_registry_plugin_v1beta1_compression_proto_init() {
	if File_buf_registry_plugin_v1beta1_compression_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_buf_registry_plugin_v1beta1_compression_proto_rawDesc), len(file_buf_registry_plugin_v1beta1_compression_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   0,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_buf_registry_plugin_v1beta1_compression_proto_goTypes,
		DependencyIndexes: file_buf_registry_plugin_v1beta1_compression_proto_depIdxs,
		EnumInfos:         file_buf_registry_plugin_v1beta1_compression_proto_enumTypes,
	}.Build()
	File_buf_registry_plugin_v1beta1_compression_proto = out.File
	file_buf_registry_plugin_v1beta1_compression_proto_goTypes = nil
	file_buf_registry_plugin_v1beta1_compression_proto_depIdxs = nil
}
