// SPDX-FileCopyrightText: 2021 The Go Language Server Authors
// SPDX-License-Identifier: BSD-3-Clause

package protocol

// ClientCapabilitiesShowDocument alias of ShowDocumentClientCapabilities.
//
// Deprecated: Use ShowDocumentClientCapabilities instead.
type ClientCapabilitiesShowDocument = ShowDocumentClientCapabilities

// ClientCapabilitiesShowMessageRequest alias of ShowMessageRequestClientCapabilities.
//
// Deprecated: Use ShowMessageRequestClientCapabilities instead.
type ClientCapabilitiesShowMessageRequest = ShowMessageRequestClientCapabilities

// ClientCapabilitiesShowMessageRequestMessageActionItem alias of ShowMessageRequestClientCapabilitiesMessageActionItem.
//
// Deprecated: Use ShowMessageRequestClientCapabilitiesMessageActionItem instead.
type ClientCapabilitiesShowMessageRequestMessageActionItem = ShowMessageRequestClientCapabilitiesMessageActionItem

// ReferencesParams alias of ReferenceParams.
//
// Deprecated: Use ReferenceParams instead.
type ReferencesParams = ReferenceParams

// TextDocumentClientCapabilitiesCallHierarchy alias of CallHierarchyClientCapabilities.
//
// Deprecated: Use CallHierarchyClientCapabilities instead.
type TextDocumentClientCapabilitiesCallHierarchy = CallHierarchyClientCapabilities

// TextDocumentClientCapabilitiesCodeAction alias of CodeActionClientCapabilities.
//
// Deprecated: Use CodeActionClientCapabilities instead.
type TextDocumentClientCapabilitiesCodeAction = CodeActionClientCapabilities

// TextDocumentClientCapabilitiesCodeActionKind alias of CodeActionClientCapabilitiesKind.
//
// Deprecated: Use CodeActionClientCapabilitiesKind instead.
type TextDocumentClientCapabilitiesCodeActionKind = CodeActionClientCapabilitiesKind

// TextDocumentClientCapabilitiesCodeActionLiteralSupport alias of CodeActionClientCapabilitiesLiteralSupport.
//
// Deprecated: Use CodeActionClientCapabilitiesLiteralSupport instead.
type TextDocumentClientCapabilitiesCodeActionLiteralSupport = CodeActionClientCapabilitiesLiteralSupport

// TextDocumentClientCapabilitiesCodeActionResolveSupport alias of CodeActionClientCapabilitiesResolveSupport.
//
// Deprecated: Use CodeActionClientCapabilitiesResolveSupport instead.
type TextDocumentClientCapabilitiesCodeActionResolveSupport = CodeActionClientCapabilitiesResolveSupport

// TextDocumentClientCapabilitiesCodeLens alias of CodeLensClientCapabilities.
//
// Deprecated: Use CodeLensClientCapabilities instead.
type TextDocumentClientCapabilitiesCodeLens = CodeLensClientCapabilities

// TextDocumentClientCapabilitiesColorProvider alias of DocumentColorClientCapabilities.
//
// Deprecated: Use DocumentColorClientCapabilities instead.
type TextDocumentClientCapabilitiesColorProvider = DocumentColorClientCapabilities

// TextDocumentClientCapabilitiesCompletion alias of CompletionTextDocumentClientCapabilities.
//
// Deprecated: Use CompletionTextDocumentClientCapabilities instead.
type TextDocumentClientCapabilitiesCompletion = CompletionTextDocumentClientCapabilities

// TextDocumentClientCapabilitiesCompletionItem alias of CompletionTextDocumentClientCapabilitiesItem.
//
// Deprecated: Use CompletionTextDocumentClientCapabilitiesItem instead.
type TextDocumentClientCapabilitiesCompletionItem = CompletionTextDocumentClientCapabilitiesItem

// TextDocumentClientCapabilitiesCompletionItemInsertTextModeSupport alias of CompletionTextDocumentClientCapabilitiesItemInsertTextModeSupport.
//
// Deprecated: Use CompletionTextDocumentClientCapabilitiesItemInsertTextModeSupport instead.
type TextDocumentClientCapabilitiesCompletionItemInsertTextModeSupport = CompletionTextDocumentClientCapabilitiesItemInsertTextModeSupport

// TextDocumentClientCapabilitiesCompletionItemKind alias of CompletionTextDocumentClientCapabilitiesItemKind.
//
// Deprecated: Use CompletionTextDocumentClientCapabilitiesItemKind instead.
type TextDocumentClientCapabilitiesCompletionItemKind = CompletionTextDocumentClientCapabilitiesItemKind

// TextDocumentClientCapabilitiesCompletionItemResolveSupport alias of CompletionTextDocumentClientCapabilitiesItemResolveSupport.
//
// Deprecated: Use CompletionTextDocumentClientCapabilitiesItemResolveSupport instead.
type TextDocumentClientCapabilitiesCompletionItemResolveSupport = CompletionTextDocumentClientCapabilitiesItemResolveSupport

// TextDocumentClientCapabilitiesCompletionItemTagSupport alias of CompletionTextDocumentClientCapabilitiesItemTagSupport.
//
// Deprecated: Use CompletionTextDocumentClientCapabilitiesItemTagSupport instead.
type TextDocumentClientCapabilitiesCompletionItemTagSupport = CompletionTextDocumentClientCapabilitiesItemTagSupport

// TextDocumentClientCapabilitiesDeclaration alias of DeclarationTextDocumentClientCapabilities.
//
// Deprecated: Use DeclarationTextDocumentClientCapabilities instead.
type TextDocumentClientCapabilitiesDeclaration = DeclarationTextDocumentClientCapabilities

// TextDocumentClientCapabilitiesDefinition alias of DefinitionTextDocumentClientCapabilities.
//
// Deprecated: Use DefinitionTextDocumentClientCapabilities instead.
type TextDocumentClientCapabilitiesDefinition = DefinitionTextDocumentClientCapabilities

// TextDocumentClientCapabilitiesDocumentHighlight alias of DocumentHighlightClientCapabilities.
//
// Deprecated: Use DocumentHighlightClientCapabilities instead.
type TextDocumentClientCapabilitiesDocumentHighlight = DocumentHighlightClientCapabilities

// TextDocumentClientCapabilitiesDocumentLink alias of DocumentLinkClientCapabilities.
//
// Deprecated: Use DocumentLinkClientCapabilities instead.
type TextDocumentClientCapabilitiesDocumentLink = DocumentLinkClientCapabilities

// TextDocumentClientCapabilitiesDocumentSymbol alias of DocumentSymbolClientCapabilities.
//
// Deprecated: Use DocumentSymbolClientCapabilities instead.
type TextDocumentClientCapabilitiesDocumentSymbol = DocumentSymbolClientCapabilities

// TextDocumentClientCapabilitiesDocumentSymbolTagSupport alias of DocumentSymbolClientCapabilitiesTagSupport.
//
// Deprecated: Use DocumentSymbolClientCapabilitiesTagSupport instead.
type TextDocumentClientCapabilitiesDocumentSymbolTagSupport = DocumentSymbolClientCapabilitiesTagSupport

// TextDocumentClientCapabilitiesFoldingRange alias of FoldingRangeClientCapabilities.
//
// Deprecated: Use FoldingRangeClientCapabilities instead.
type TextDocumentClientCapabilitiesFoldingRange = FoldingRangeClientCapabilities

// TextDocumentClientCapabilitiesFormatting alias of DocumentFormattingClientCapabilities.
//
// Deprecated: Use DocumentFormattingClientCapabilities instead.
type TextDocumentClientCapabilitiesFormatting = DocumentFormattingClientCapabilities

// TextDocumentClientCapabilitiesHover alias of HoverTextDocumentClientCapabilities.
//
// Deprecated: Use HoverTextDocumentClientCapabilities instead.
type TextDocumentClientCapabilitiesHover = HoverTextDocumentClientCapabilities

// TextDocumentClientCapabilitiesImplementation alias of ImplementationTextDocumentClientCapabilities.
//
// Deprecated: Use ImplementationTextDocumentClientCapabilities instead.
type TextDocumentClientCapabilitiesImplementation = ImplementationTextDocumentClientCapabilities

// TextDocumentClientCapabilitiesLinkedEditingRange alias of LinkedEditingRangeClientCapabilities.
//
// Deprecated: Use LinkedEditingRangeClientCapabilities instead.
type TextDocumentClientCapabilitiesLinkedEditingRange = LinkedEditingRangeClientCapabilities

// TextDocumentClientCapabilitiesMoniker of MonikerClientCapabilities.
//
// Deprecated: Use MonikerClientCapabilities instead.
type TextDocumentClientCapabilitiesMoniker = MonikerClientCapabilities

// TextDocumentClientCapabilitiesOnTypeFormatting of DocumentOnTypeFormattingClientCapabilities.
//
// Deprecated: Use DocumentOnTypeFormattingClientCapabilities instead.
type TextDocumentClientCapabilitiesOnTypeFormatting = DocumentOnTypeFormattingClientCapabilities

// TextDocumentClientCapabilitiesPublishDiagnostics of PublishDiagnosticsClientCapabilities.
//
// Deprecated: Use PublishDiagnosticsClientCapabilities instead.
type TextDocumentClientCapabilitiesPublishDiagnostics = PublishDiagnosticsClientCapabilities

// TextDocumentClientCapabilitiesPublishDiagnosticsTagSupport of PublishDiagnosticsClientCapabilitiesTagSupport.
//
// Deprecated: Use PublishDiagnosticsClientCapabilitiesTagSupport instead.
type TextDocumentClientCapabilitiesPublishDiagnosticsTagSupport = PublishDiagnosticsClientCapabilitiesTagSupport

// TextDocumentClientCapabilitiesRangeFormatting of DocumentRangeFormattingClientCapabilities.
//
// Deprecated: Use DocumentRangeFormattingClientCapabilities instead.
type TextDocumentClientCapabilitiesRangeFormatting = DocumentRangeFormattingClientCapabilities

// TextDocumentClientCapabilitiesReferences of ReferencesTextDocumentClientCapabilities.
//
// Deprecated: Use ReferencesTextDocumentClientCapabilities instead.
type TextDocumentClientCapabilitiesReferences = ReferencesTextDocumentClientCapabilities

// TextDocumentClientCapabilitiesRename of RenameClientCapabilities.
//
// Deprecated: Use RenameClientCapabilities instead.
type TextDocumentClientCapabilitiesRename = RenameClientCapabilities

// TextDocumentClientCapabilitiesSelectionRange of SelectionRangeClientCapabilities.
//
// Deprecated: Use SelectionRangeClientCapabilities instead.
type TextDocumentClientCapabilitiesSelectionRange = SelectionRangeClientCapabilities

// TextDocumentClientCapabilitiesSemanticTokens of SemanticTokensClientCapabilities.
//
// Deprecated: Use SemanticTokensClientCapabilities instead.
type TextDocumentClientCapabilitiesSemanticTokens = SemanticTokensClientCapabilities

// TextDocumentClientCapabilitiesSignatureHelp of SignatureHelpTextDocumentClientCapabilities.
//
// Deprecated: Use SignatureHelpTextDocumentClientCapabilities instead.
type TextDocumentClientCapabilitiesSignatureHelp = SignatureHelpTextDocumentClientCapabilities

// TextDocumentClientCapabilitiesSynchronization of TextDocumentSyncClientCapabilities.
//
// Deprecated: Use TextDocumentSyncClientCapabilities instead.
type TextDocumentClientCapabilitiesSynchronization = TextDocumentSyncClientCapabilities

// TextDocumentClientCapabilitiesTypeDefinition of TypeDefinitionTextDocumentClientCapabilities.
//
// Deprecated: Use TypeDefinitionTextDocumentClientCapabilities instead.
type TextDocumentClientCapabilitiesTypeDefinition = TypeDefinitionTextDocumentClientCapabilities

// Abort alias of FailureHandlingKindAbort.
//
// Deprecated: Use FailureHandlingKindAbort instead.
const Abort = FailureHandlingKindAbort

// TextOnlyTransactional alias of FailureHandlingKindTextOnlyTransactional.
//
// Deprecated: Use FailureHandlingKindTextOnlyTransactional instead.
const TextOnlyTransactional = FailureHandlingKindTextOnlyTransactional

// Transactional alias of FailureHandlingKindTransactional.
//
// Deprecated: Use FailureHandlingKindTransactional instead.
const Transactional = FailureHandlingKindTransactional

// Undo alias of FailureHandlingKindUndo.
//
// Deprecated: Use FailureHandlingKindUndo instead.
const Undo = FailureHandlingKindUndo

// WorkspaceClientCapabilitiesSymbol alias of WorkspaceSymbolClientCapabilities.
//
// Deprecated: Use WorkspaceSymbolClientCapabilities instead.
type WorkspaceClientCapabilitiesSymbol = WorkspaceSymbolClientCapabilities

// WorkspaceClientCapabilitiesSymbolKind alias of SymbolKindCapabilities.
//
// Deprecated: Use SymbolKindCapabilities instead.
type WorkspaceClientCapabilitiesSymbolKind = SymbolKindCapabilities

// WorkspaceClientCapabilitiesCodeLens alias of CodeLensWorkspaceClientCapabilities.
//
// Deprecated: Use CodeLensWorkspaceClientCapabilities instead.
type WorkspaceClientCapabilitiesCodeLens = CodeLensWorkspaceClientCapabilities

// WorkspaceClientCapabilitiesDidChangeConfiguration alias of DidChangeConfigurationWorkspaceClientCapabilities.
//
// Deprecated: Use DidChangeConfigurationWorkspaceClientCapabilities instead.
type WorkspaceClientCapabilitiesDidChangeConfiguration = DidChangeConfigurationWorkspaceClientCapabilities

// WorkspaceClientCapabilitiesDidChangeWatchedFiles alias of DidChangeWatchedFilesWorkspaceClientCapabilities.
//
// Deprecated: Use DidChangeWatchedFilesWorkspaceClientCapabilities instead.
type WorkspaceClientCapabilitiesDidChangeWatchedFiles = DidChangeWatchedFilesWorkspaceClientCapabilities

// WorkspaceClientCapabilitiesExecuteCommand alias of ExecuteCommandClientCapabilities.
//
// Deprecated: Use ExecuteCommandClientCapabilities instead.
type WorkspaceClientCapabilitiesExecuteCommand = ExecuteCommandClientCapabilities

// WorkspaceClientCapabilitiesSemanticTokens alias of SemanticTokensWorkspaceClientCapabilities.
//
// Deprecated: Use SemanticTokensWorkspaceClientCapabilities instead.
type WorkspaceClientCapabilitiesSemanticTokens = SemanticTokensWorkspaceClientCapabilities

// WorkspaceClientCapabilitiesSemanticTokensRequests alias of SemanticTokensWorkspaceClientCapabilitiesRequests.
//
// Deprecated: Use SemanticTokensWorkspaceClientCapabilitiesRequests instead.
type WorkspaceClientCapabilitiesSemanticTokensRequests = SemanticTokensWorkspaceClientCapabilitiesRequests
