// Copyright 2025 Buf Technologies, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        (unknown)
// source: buf/descriptor/v1/module_name.proto

//go:build !protoopaque

package descriptorv1

import (
	_ "buf.build/gen/go/bufbuild/protovalidate/protocolbuffers/go/buf/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The fully-qualified name of a module on the Buf Schema Registry.
type ModuleName struct {
	state protoimpl.MessageState `protogen:"hybrid.v1"`
	// The registry hostname of the module.
	Registry string `protobuf:"bytes,1,opt,name=registry,proto3" json:"registry,omitempty"`
	// The name of the owner of the module, either a user or organization.
	Owner string `protobuf:"bytes,2,opt,name=owner,proto3" json:"owner,omitempty"`
	// The name of the module.
	Module        string `protobuf:"bytes,3,opt,name=module,proto3" json:"module,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ModuleName) Reset() {
	*x = ModuleName{}
	mi := &file_buf_descriptor_v1_module_name_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ModuleName) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ModuleName) ProtoMessage() {}

func (x *ModuleName) ProtoReflect() protoreflect.Message {
	mi := &file_buf_descriptor_v1_module_name_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

func (x *ModuleName) GetRegistry() string {
	if x != nil {
		return x.Registry
	}
	return ""
}

func (x *ModuleName) GetOwner() string {
	if x != nil {
		return x.Owner
	}
	return ""
}

func (x *ModuleName) GetModule() string {
	if x != nil {
		return x.Module
	}
	return ""
}

func (x *ModuleName) SetRegistry(v string) {
	x.Registry = v
}

func (x *ModuleName) SetOwner(v string) {
	x.Owner = v
}

func (x *ModuleName) SetModule(v string) {
	x.Module = v
}

type ModuleName_builder struct {
	_ [0]func() // Prevents comparability and use of unkeyed literals for the builder.

	// The registry hostname of the module.
	Registry string
	// The name of the owner of the module, either a user or organization.
	Owner string
	// The name of the module.
	Module string
}

func (b0 ModuleName_builder) Build() *ModuleName {
	m0 := &ModuleName{}
	b, x := &b0, m0
	_, _ = b, x
	x.Registry = b.Registry
	x.Owner = b.Owner
	x.Module = b.Module
	return m0
}

var File_buf_descriptor_v1_module_name_proto protoreflect.FileDescriptor

const file_buf_descriptor_v1_module_name_proto_rawDesc = "" +
	"\n" +
	"#buf/descriptor/v1/module_name.proto\x12\x11buf.descriptor.v1\x1a\x1bbuf/validate/validate.proto\"u\n" +
	"\n" +
	"ModuleName\x12\"\n" +
	"\bregistry\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\bregistry\x12 \n" +
	"\x05owner\x18\x02 \x01(\tB\n" +
	"\xbaH\a\xc8\x01\x01r\x02\x18 R\x05owner\x12!\n" +
	"\x06module\x18\x03 \x01(\tB\t\xbaH\x06r\x04\x10\x02\x18dR\x06moduleB]Z[buf.build/gen/go/bufbuild/protodescriptor/protocolbuffers/go/buf/descriptor/v1;descriptorv1b\x06proto3"

var file_buf_descriptor_v1_module_name_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_buf_descriptor_v1_module_name_proto_goTypes = []any{
	(*ModuleName)(nil), // 0: buf.descriptor.v1.ModuleName
}
var file_buf_descriptor_v1_module_name_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_buf_descriptor_v1_module_name_proto_init() }
func file_buf_descriptor_v1_module_name_proto_init() {
	if File_buf_descriptor_v1_module_name_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_buf_descriptor_v1_module_name_proto_rawDesc), len(file_buf_descriptor_v1_module_name_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_buf_descriptor_v1_module_name_proto_goTypes,
		DependencyIndexes: file_buf_descriptor_v1_module_name_proto_depIdxs,
		MessageInfos:      file_buf_descriptor_v1_module_name_proto_msgTypes,
	}.Build()
	File_buf_descriptor_v1_module_name_proto = out.File
	file_buf_descriptor_v1_module_name_proto_goTypes = nil
	file_buf_descriptor_v1_module_name_proto_depIdxs = nil
}
