// Copyright 2025-2026 Buf Technologies, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package appcmd

// invalidArgumentError is used to indicate that an error was
// caused by argument validation.
type invalidArgumentError struct {
	err error
}

func newInvalidArgumentError(err error) *invalidArgumentError {
	return &invalidArgumentError{err: err}
}

func (a *invalidArgumentError) Error() string {
	if a == nil {
		return ""
	}
	return a.err.Error()
}

func (a *invalidArgumentError) Unwrap() error {
	if a == nil {
		return nil
	}
	return a.err
}
