// Copyright 2025-2026 Buf Technologies, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package app

import (
	"io"
)

type stderrContainer struct {
	writer io.Writer
}

func newStderrContainer(writer io.Writer) *stderrContainer {
	if writer == nil {
		writer = io.Discard
	}
	return &stderrContainer{
		writer: newLockedWriter(writer),
	}
}

func (s *stderrContainer) Stderr() io.Writer {
	return s.writer
}
