// Copyright 2020-2026 Buf Technologies, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package bufcli

import (
	"context"

	"github.com/bufbuild/buf/private/bufpkg/bufconfig"
)

// GetBufWorkYAMLFileForDirPath gets the buf.work.yaml file for the directory path.
func GetBufWorkYAMLFileForDirPath(
	ctx context.Context,
	dirPath string,
) (bufconfig.BufWorkYAMLFile, error) {
	bucket, err := newOSReadWriteBucketWithSymlinks(dirPath)
	if err != nil {
		return nil, err
	}
	return bufconfig.GetBufWorkYAMLFileForPrefix(ctx, bucket, ".")
}
