// Copyright 2020-2026 Buf Technologies, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package git

import "encoding/json"

type refWithBranch struct {
	ref    string
	branch string
}

func newRefWithBranch(ref string, branch string) *refWithBranch {
	return &refWithBranch{
		ref:    ref,
		branch: branch,
	}
}

func (r *refWithBranch) cloneBranch() string {
	if r == nil {
		return ""
	}
	return r.branch
}

func (r *refWithBranch) checkout() string {
	if r == nil {
		return ""
	}
	return r.ref
}

func (r *refWithBranch) MarshalJSON() ([]byte, error) {
	return json.Marshal(struct {
		Ref    string
		Branch string
	}{
		Ref:    r.checkout(),
		Branch: r.cloneBranch(),
	})
}

func (r *refWithBranch) String() string {
	return r.checkout()
}
