// Copyright 2025 Buf Technologies, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        (unknown)
// source: buf/descriptor/v1/descriptor.proto

// Interfaces that extend types from the google.protobuf package.

//go:build !protoopaque

package descriptorv1

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	descriptorpb "google.golang.org/protobuf/types/descriptorpb"
	reflect "reflect"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

var file_buf_descriptor_v1_descriptor_proto_extTypes = []protoimpl.ExtensionInfo{
	{
		ExtendedType:  (*descriptorpb.FileDescriptorSet)(nil),
		ExtensionType: (*FileDescriptorSetExtension)(nil),
		Field:         536000000,
		Name:          "buf.descriptor.v1.buf_file_descriptor_set_extension",
		Tag:           "bytes,536000000,opt,name=buf_file_descriptor_set_extension",
		Filename:      "buf/descriptor/v1/descriptor.proto",
	},
	{
		ExtendedType:  (*descriptorpb.SourceCodeInfo)(nil),
		ExtensionType: (*SourceCodeInfoExtension)(nil),
		Field:         536000000,
		Name:          "buf.descriptor.v1.buf_source_code_info_extension",
		Tag:           "bytes,536000000,opt,name=buf_source_code_info_extension",
		Filename:      "buf/descriptor/v1/descriptor.proto",
	},
}

// Extension fields to descriptorpb.FileDescriptorSet.
var (
	// Additional properties of a FileDescriptorSet.
	//
	// optional buf.descriptor.v1.FileDescriptorSetExtension buf_file_descriptor_set_extension = 536000000;
	E_BufFileDescriptorSetExtension = &file_buf_descriptor_v1_descriptor_proto_extTypes[0]
)

// Extension fields to descriptorpb.SourceCodeInfo.
var (
	// Additional properties of a SourceCodeInfo.
	//
	// optional buf.descriptor.v1.SourceCodeInfoExtension buf_source_code_info_extension = 536000000;
	E_BufSourceCodeInfoExtension = &file_buf_descriptor_v1_descriptor_proto_extTypes[1]
)

var File_buf_descriptor_v1_descriptor_proto protoreflect.FileDescriptor

const file_buf_descriptor_v1_descriptor_proto_rawDesc = "" +
	"\n" +
	"\"buf/descriptor/v1/descriptor.proto\x12\x11buf.descriptor.v1\x1a5buf/descriptor/v1/file_descriptor_set_extension.proto\x1a2buf/descriptor/v1/source_code_info_extension.proto\x1a google/protobuf/descriptor.proto:\x9f\x01\n" +
	"!buf_file_descriptor_set_extension\x12\".google.protobuf.FileDescriptorSet\x18\x80\xec\xca\xff\x01 \x01(\v2-.buf.descriptor.v1.FileDescriptorSetExtensionR\x1dbufFileDescriptorSetExtension:\x93\x01\n" +
	"\x1ebuf_source_code_info_extension\x12\x1f.google.protobuf.SourceCodeInfo\x18\x80\xec\xca\xff\x01 \x01(\v2*.buf.descriptor.v1.SourceCodeInfoExtensionR\x1abufSourceCodeInfoExtensionB]Z[buf.build/gen/go/bufbuild/protodescriptor/protocolbuffers/go/buf/descriptor/v1;descriptorv1"

var file_buf_descriptor_v1_descriptor_proto_goTypes = []any{
	(*descriptorpb.FileDescriptorSet)(nil), // 0: google.protobuf.FileDescriptorSet
	(*descriptorpb.SourceCodeInfo)(nil),    // 1: google.protobuf.SourceCodeInfo
	(*FileDescriptorSetExtension)(nil),     // 2: buf.descriptor.v1.FileDescriptorSetExtension
	(*SourceCodeInfoExtension)(nil),        // 3: buf.descriptor.v1.SourceCodeInfoExtension
}
var file_buf_descriptor_v1_descriptor_proto_depIdxs = []int32{
	0, // 0: buf.descriptor.v1.buf_file_descriptor_set_extension:extendee -> google.protobuf.FileDescriptorSet
	1, // 1: buf.descriptor.v1.buf_source_code_info_extension:extendee -> google.protobuf.SourceCodeInfo
	2, // 2: buf.descriptor.v1.buf_file_descriptor_set_extension:type_name -> buf.descriptor.v1.FileDescriptorSetExtension
	3, // 3: buf.descriptor.v1.buf_source_code_info_extension:type_name -> buf.descriptor.v1.SourceCodeInfoExtension
	4, // [4:4] is the sub-list for method output_type
	4, // [4:4] is the sub-list for method input_type
	2, // [2:4] is the sub-list for extension type_name
	0, // [0:2] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_buf_descriptor_v1_descriptor_proto_init() }
func file_buf_descriptor_v1_descriptor_proto_init() {
	if File_buf_descriptor_v1_descriptor_proto != nil {
		return
	}
	file_buf_descriptor_v1_file_descriptor_set_extension_proto_init()
	file_buf_descriptor_v1_source_code_info_extension_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_buf_descriptor_v1_descriptor_proto_rawDesc), len(file_buf_descriptor_v1_descriptor_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   0,
			NumExtensions: 2,
			NumServices:   0,
		},
		GoTypes:           file_buf_descriptor_v1_descriptor_proto_goTypes,
		DependencyIndexes: file_buf_descriptor_v1_descriptor_proto_depIdxs,
		ExtensionInfos:    file_buf_descriptor_v1_descriptor_proto_extTypes,
	}.Build()
	File_buf_descriptor_v1_descriptor_proto = out.File
	file_buf_descriptor_v1_descriptor_proto_goTypes = nil
	file_buf_descriptor_v1_descriptor_proto_depIdxs = nil
}
