// Copyright 2020-2026 Buf Technologies, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package internal

import (
	"sort"
	"strings"
)

func normalizeFormat(format string) string {
	return strings.ToLower(strings.TrimSpace(format))
}

func formatsToString(formats map[string]struct{}) string {
	if len(formats) == 0 {
		return "[]"
	}
	s := make([]string, 0, len(formats))
	for format := range formats {
		s = append(s, format)
	}
	sort.Strings(s)
	return "[" + strings.Join(s, ",") + "]"
}
