// Copyright 2020-2026 Buf Technologies, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        (unknown)
// source: buf/alpha/registry/v1alpha1/role.proto

package registryv1alpha1

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The roles that users can have in a Server.
type ServerRole int32

const (
	ServerRole_SERVER_ROLE_UNSPECIFIED ServerRole = 0
	ServerRole_SERVER_ROLE_ADMIN       ServerRole = 1
	ServerRole_SERVER_ROLE_MEMBER      ServerRole = 2
)

// Enum value maps for ServerRole.
var (
	ServerRole_name = map[int32]string{
		0: "SERVER_ROLE_UNSPECIFIED",
		1: "SERVER_ROLE_ADMIN",
		2: "SERVER_ROLE_MEMBER",
	}
	ServerRole_value = map[string]int32{
		"SERVER_ROLE_UNSPECIFIED": 0,
		"SERVER_ROLE_ADMIN":       1,
		"SERVER_ROLE_MEMBER":      2,
	}
)

func (x ServerRole) Enum() *ServerRole {
	p := new(ServerRole)
	*p = x
	return p
}

func (x ServerRole) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ServerRole) Descriptor() protoreflect.EnumDescriptor {
	return file_buf_alpha_registry_v1alpha1_role_proto_enumTypes[0].Descriptor()
}

func (ServerRole) Type() protoreflect.EnumType {
	return &file_buf_alpha_registry_v1alpha1_role_proto_enumTypes[0]
}

func (x ServerRole) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// The roles that users can have in a Organization.
type OrganizationRole int32

const (
	OrganizationRole_ORGANIZATION_ROLE_UNSPECIFIED OrganizationRole = 0
	OrganizationRole_ORGANIZATION_ROLE_OWNER       OrganizationRole = 1
	OrganizationRole_ORGANIZATION_ROLE_ADMIN       OrganizationRole = 2
	OrganizationRole_ORGANIZATION_ROLE_MEMBER      OrganizationRole = 3
	OrganizationRole_ORGANIZATION_ROLE_MACHINE     OrganizationRole = 4
)

// Enum value maps for OrganizationRole.
var (
	OrganizationRole_name = map[int32]string{
		0: "ORGANIZATION_ROLE_UNSPECIFIED",
		1: "ORGANIZATION_ROLE_OWNER",
		2: "ORGANIZATION_ROLE_ADMIN",
		3: "ORGANIZATION_ROLE_MEMBER",
		4: "ORGANIZATION_ROLE_MACHINE",
	}
	OrganizationRole_value = map[string]int32{
		"ORGANIZATION_ROLE_UNSPECIFIED": 0,
		"ORGANIZATION_ROLE_OWNER":       1,
		"ORGANIZATION_ROLE_ADMIN":       2,
		"ORGANIZATION_ROLE_MEMBER":      3,
		"ORGANIZATION_ROLE_MACHINE":     4,
	}
)

func (x OrganizationRole) Enum() *OrganizationRole {
	p := new(OrganizationRole)
	*p = x
	return p
}

func (x OrganizationRole) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (OrganizationRole) Descriptor() protoreflect.EnumDescriptor {
	return file_buf_alpha_registry_v1alpha1_role_proto_enumTypes[1].Descriptor()
}

func (OrganizationRole) Type() protoreflect.EnumType {
	return &file_buf_alpha_registry_v1alpha1_role_proto_enumTypes[1]
}

func (x OrganizationRole) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// The source of a user's role in an Organization.
type OrganizationRoleSource int32

const (
	OrganizationRoleSource_ORGANIZATION_ROLE_SOURCE_UNSPECIFIED OrganizationRoleSource = 0
	OrganizationRoleSource_ORGANIZATION_ROLE_SOURCE_DIRECT      OrganizationRoleSource = 1
	OrganizationRoleSource_ORGANIZATION_ROLE_SOURCE_JIT         OrganizationRoleSource = 2
	OrganizationRoleSource_ORGANIZATION_ROLE_SOURCE_IDP_GROUP   OrganizationRoleSource = 3
)

// Enum value maps for OrganizationRoleSource.
var (
	OrganizationRoleSource_name = map[int32]string{
		0: "ORGANIZATION_ROLE_SOURCE_UNSPECIFIED",
		1: "ORGANIZATION_ROLE_SOURCE_DIRECT",
		2: "ORGANIZATION_ROLE_SOURCE_JIT",
		3: "ORGANIZATION_ROLE_SOURCE_IDP_GROUP",
	}
	OrganizationRoleSource_value = map[string]int32{
		"ORGANIZATION_ROLE_SOURCE_UNSPECIFIED": 0,
		"ORGANIZATION_ROLE_SOURCE_DIRECT":      1,
		"ORGANIZATION_ROLE_SOURCE_JIT":         2,
		"ORGANIZATION_ROLE_SOURCE_IDP_GROUP":   3,
	}
)

func (x OrganizationRoleSource) Enum() *OrganizationRoleSource {
	p := new(OrganizationRoleSource)
	*p = x
	return p
}

func (x OrganizationRoleSource) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (OrganizationRoleSource) Descriptor() protoreflect.EnumDescriptor {
	return file_buf_alpha_registry_v1alpha1_role_proto_enumTypes[2].Descriptor()
}

func (OrganizationRoleSource) Type() protoreflect.EnumType {
	return &file_buf_alpha_registry_v1alpha1_role_proto_enumTypes[2]
}

func (x OrganizationRoleSource) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// The roles that users can have for a Repository.
type RepositoryRole int32

const (
	RepositoryRole_REPOSITORY_ROLE_UNSPECIFIED   RepositoryRole = 0
	RepositoryRole_REPOSITORY_ROLE_OWNER         RepositoryRole = 1
	RepositoryRole_REPOSITORY_ROLE_ADMIN         RepositoryRole = 2
	RepositoryRole_REPOSITORY_ROLE_WRITE         RepositoryRole = 3
	RepositoryRole_REPOSITORY_ROLE_READ          RepositoryRole = 4
	RepositoryRole_REPOSITORY_ROLE_LIMITED_WRITE RepositoryRole = 5
)

// Enum value maps for RepositoryRole.
var (
	RepositoryRole_name = map[int32]string{
		0: "REPOSITORY_ROLE_UNSPECIFIED",
		1: "REPOSITORY_ROLE_OWNER",
		2: "REPOSITORY_ROLE_ADMIN",
		3: "REPOSITORY_ROLE_WRITE",
		4: "REPOSITORY_ROLE_READ",
		5: "REPOSITORY_ROLE_LIMITED_WRITE",
	}
	RepositoryRole_value = map[string]int32{
		"REPOSITORY_ROLE_UNSPECIFIED":   0,
		"REPOSITORY_ROLE_OWNER":         1,
		"REPOSITORY_ROLE_ADMIN":         2,
		"REPOSITORY_ROLE_WRITE":         3,
		"REPOSITORY_ROLE_READ":          4,
		"REPOSITORY_ROLE_LIMITED_WRITE": 5,
	}
)

func (x RepositoryRole) Enum() *RepositoryRole {
	p := new(RepositoryRole)
	*p = x
	return p
}

func (x RepositoryRole) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (RepositoryRole) Descriptor() protoreflect.EnumDescriptor {
	return file_buf_alpha_registry_v1alpha1_role_proto_enumTypes[3].Descriptor()
}

func (RepositoryRole) Type() protoreflect.EnumType {
	return &file_buf_alpha_registry_v1alpha1_role_proto_enumTypes[3]
}

func (x RepositoryRole) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// The source of a user's role in a Repository.
type RepositoryRoleSource int32

const (
	RepositoryRoleSource_REPOSITORY_ROLE_SOURCE_UNSPECIFIED RepositoryRoleSource = 0
	RepositoryRoleSource_REPOSITORY_ROLE_SOURCE_DIRECT      RepositoryRoleSource = 1
	RepositoryRoleSource_REPOSITORY_ROLE_SOURCE_IDP_GROUP   RepositoryRoleSource = 2
)

// Enum value maps for RepositoryRoleSource.
var (
	RepositoryRoleSource_name = map[int32]string{
		0: "REPOSITORY_ROLE_SOURCE_UNSPECIFIED",
		1: "REPOSITORY_ROLE_SOURCE_DIRECT",
		2: "REPOSITORY_ROLE_SOURCE_IDP_GROUP",
	}
	RepositoryRoleSource_value = map[string]int32{
		"REPOSITORY_ROLE_SOURCE_UNSPECIFIED": 0,
		"REPOSITORY_ROLE_SOURCE_DIRECT":      1,
		"REPOSITORY_ROLE_SOURCE_IDP_GROUP":   2,
	}
)

func (x RepositoryRoleSource) Enum() *RepositoryRoleSource {
	p := new(RepositoryRoleSource)
	*p = x
	return p
}

func (x RepositoryRoleSource) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (RepositoryRoleSource) Descriptor() protoreflect.EnumDescriptor {
	return file_buf_alpha_registry_v1alpha1_role_proto_enumTypes[4].Descriptor()
}

func (RepositoryRoleSource) Type() protoreflect.EnumType {
	return &file_buf_alpha_registry_v1alpha1_role_proto_enumTypes[4]
}

func (x RepositoryRoleSource) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

var File_buf_alpha_registry_v1alpha1_role_proto protoreflect.FileDescriptor

const file_buf_alpha_registry_v1alpha1_role_proto_rawDesc = "" +
	"\n" +
	"&buf/alpha/registry/v1alpha1/role.proto\x12\x1bbuf.alpha.registry.v1alpha1*X\n" +
	"\n" +
	"ServerRole\x12\x1b\n" +
	"\x17SERVER_ROLE_UNSPECIFIED\x10\x00\x12\x15\n" +
	"\x11SERVER_ROLE_ADMIN\x10\x01\x12\x16\n" +
	"\x12SERVER_ROLE_MEMBER\x10\x02*\xac\x01\n" +
	"\x10OrganizationRole\x12!\n" +
	"\x1dORGANIZATION_ROLE_UNSPECIFIED\x10\x00\x12\x1b\n" +
	"\x17ORGANIZATION_ROLE_OWNER\x10\x01\x12\x1b\n" +
	"\x17ORGANIZATION_ROLE_ADMIN\x10\x02\x12\x1c\n" +
	"\x18ORGANIZATION_ROLE_MEMBER\x10\x03\x12\x1d\n" +
	"\x19ORGANIZATION_ROLE_MACHINE\x10\x04*\xb1\x01\n" +
	"\x16OrganizationRoleSource\x12(\n" +
	"$ORGANIZATION_ROLE_SOURCE_UNSPECIFIED\x10\x00\x12#\n" +
	"\x1fORGANIZATION_ROLE_SOURCE_DIRECT\x10\x01\x12 \n" +
	"\x1cORGANIZATION_ROLE_SOURCE_JIT\x10\x02\x12&\n" +
	"\"ORGANIZATION_ROLE_SOURCE_IDP_GROUP\x10\x03*\xbf\x01\n" +
	"\x0eRepositoryRole\x12\x1f\n" +
	"\x1bREPOSITORY_ROLE_UNSPECIFIED\x10\x00\x12\x19\n" +
	"\x15REPOSITORY_ROLE_OWNER\x10\x01\x12\x19\n" +
	"\x15REPOSITORY_ROLE_ADMIN\x10\x02\x12\x19\n" +
	"\x15REPOSITORY_ROLE_WRITE\x10\x03\x12\x18\n" +
	"\x14REPOSITORY_ROLE_READ\x10\x04\x12!\n" +
	"\x1dREPOSITORY_ROLE_LIMITED_WRITE\x10\x05*\x87\x01\n" +
	"\x14RepositoryRoleSource\x12&\n" +
	"\"REPOSITORY_ROLE_SOURCE_UNSPECIFIED\x10\x00\x12!\n" +
	"\x1dREPOSITORY_ROLE_SOURCE_DIRECT\x10\x01\x12$\n" +
	" REPOSITORY_ROLE_SOURCE_IDP_GROUP\x10\x02B\x96\x02\n" +
	"\x1fcom.buf.alpha.registry.v1alpha1B\tRoleProtoP\x01ZYgithub.com/bufbuild/buf/private/gen/proto/go/buf/alpha/registry/v1alpha1;registryv1alpha1\xa2\x02\x03BAR\xaa\x02\x1bBuf.Alpha.Registry.V1alpha1\xca\x02\x1bBuf\\Alpha\\Registry\\V1alpha1\xe2\x02'Buf\\Alpha\\Registry\\V1alpha1\\GPBMetadata\xea\x02\x1eBuf::Alpha::Registry::V1alpha1b\x06proto3"

var file_buf_alpha_registry_v1alpha1_role_proto_enumTypes = make([]protoimpl.EnumInfo, 5)
var file_buf_alpha_registry_v1alpha1_role_proto_goTypes = []any{
	(ServerRole)(0),             // 0: buf.alpha.registry.v1alpha1.ServerRole
	(OrganizationRole)(0),       // 1: buf.alpha.registry.v1alpha1.OrganizationRole
	(OrganizationRoleSource)(0), // 2: buf.alpha.registry.v1alpha1.OrganizationRoleSource
	(RepositoryRole)(0),         // 3: buf.alpha.registry.v1alpha1.RepositoryRole
	(RepositoryRoleSource)(0),   // 4: buf.alpha.registry.v1alpha1.RepositoryRoleSource
}
var file_buf_alpha_registry_v1alpha1_role_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_buf_alpha_registry_v1alpha1_role_proto_init() }
func file_buf_alpha_registry_v1alpha1_role_proto_init() {
	if File_buf_alpha_registry_v1alpha1_role_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_buf_alpha_registry_v1alpha1_role_proto_rawDesc), len(file_buf_alpha_registry_v1alpha1_role_proto_rawDesc)),
			NumEnums:      5,
			NumMessages:   0,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_buf_alpha_registry_v1alpha1_role_proto_goTypes,
		DependencyIndexes: file_buf_alpha_registry_v1alpha1_role_proto_depIdxs,
		EnumInfos:         file_buf_alpha_registry_v1alpha1_role_proto_enumTypes,
	}.Build()
	File_buf_alpha_registry_v1alpha1_role_proto = out.File
	file_buf_alpha_registry_v1alpha1_role_proto_goTypes = nil
	file_buf_alpha_registry_v1alpha1_role_proto_depIdxs = nil
}
