// Copyright 2025 Buf Technologies, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        (unknown)
// source: buf/descriptor/v1/file_descriptor_set_extension.proto

//go:build !protoopaque

package descriptorv1

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Extensions to google.protobuf.FileDescriptorSet.
//
// These are encapsulated in a single message so that a single extension number can be used
// on google.protobuf.FileDescriptorSet.
type FileDescriptorSetExtension struct {
	state protoimpl.MessageState `protogen:"hybrid.v1"`
	// Extensions to each FileDescriptorProto in the FileDescriptorSet.
	File          []*FileDescriptorProtoExtension `protobuf:"bytes,1,rep,name=file,proto3" json:"file,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *FileDescriptorSetExtension) Reset() {
	*x = FileDescriptorSetExtension{}
	mi := &file_buf_descriptor_v1_file_descriptor_set_extension_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FileDescriptorSetExtension) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FileDescriptorSetExtension) ProtoMessage() {}

func (x *FileDescriptorSetExtension) ProtoReflect() protoreflect.Message {
	mi := &file_buf_descriptor_v1_file_descriptor_set_extension_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

func (x *FileDescriptorSetExtension) GetFile() []*FileDescriptorProtoExtension {
	if x != nil {
		return x.File
	}
	return nil
}

func (x *FileDescriptorSetExtension) SetFile(v []*FileDescriptorProtoExtension) {
	x.File = v
}

type FileDescriptorSetExtension_builder struct {
	_ [0]func() // Prevents comparability and use of unkeyed literals for the builder.

	// Extensions to each FileDescriptorProto in the FileDescriptorSet.
	File []*FileDescriptorProtoExtension
}

func (b0 FileDescriptorSetExtension_builder) Build() *FileDescriptorSetExtension {
	m0 := &FileDescriptorSetExtension{}
	b, x := &b0, m0
	_, _ = b, x
	x.File = b.File
	return m0
}

var File_buf_descriptor_v1_file_descriptor_set_extension_proto protoreflect.FileDescriptor

const file_buf_descriptor_v1_file_descriptor_set_extension_proto_rawDesc = "" +
	"\n" +
	"5buf/descriptor/v1/file_descriptor_set_extension.proto\x12\x11buf.descriptor.v1\x1a7buf/descriptor/v1/file_descriptor_proto_extension.proto\"a\n" +
	"\x1aFileDescriptorSetExtension\x12C\n" +
	"\x04file\x18\x01 \x03(\v2/.buf.descriptor.v1.FileDescriptorProtoExtensionR\x04fileB]Z[buf.build/gen/go/bufbuild/protodescriptor/protocolbuffers/go/buf/descriptor/v1;descriptorv1b\x06proto3"

var file_buf_descriptor_v1_file_descriptor_set_extension_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_buf_descriptor_v1_file_descriptor_set_extension_proto_goTypes = []any{
	(*FileDescriptorSetExtension)(nil),   // 0: buf.descriptor.v1.FileDescriptorSetExtension
	(*FileDescriptorProtoExtension)(nil), // 1: buf.descriptor.v1.FileDescriptorProtoExtension
}
var file_buf_descriptor_v1_file_descriptor_set_extension_proto_depIdxs = []int32{
	1, // 0: buf.descriptor.v1.FileDescriptorSetExtension.file:type_name -> buf.descriptor.v1.FileDescriptorProtoExtension
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_buf_descriptor_v1_file_descriptor_set_extension_proto_init() }
func file_buf_descriptor_v1_file_descriptor_set_extension_proto_init() {
	if File_buf_descriptor_v1_file_descriptor_set_extension_proto != nil {
		return
	}
	file_buf_descriptor_v1_file_descriptor_proto_extension_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_buf_descriptor_v1_file_descriptor_set_extension_proto_rawDesc), len(file_buf_descriptor_v1_file_descriptor_set_extension_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_buf_descriptor_v1_file_descriptor_set_extension_proto_goTypes,
		DependencyIndexes: file_buf_descriptor_v1_file_descriptor_set_extension_proto_depIdxs,
		MessageInfos:      file_buf_descriptor_v1_file_descriptor_set_extension_proto_msgTypes,
	}.Build()
	File_buf_descriptor_v1_file_descriptor_set_extension_proto = out.File
	file_buf_descriptor_v1_file_descriptor_set_extension_proto_goTypes = nil
	file_buf_descriptor_v1_file_descriptor_set_extension_proto_depIdxs = nil
}
