// Copyright 2023-2025 Buf Technologies, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        (unknown)
// source: buf/registry/owner/v1/owner_service.proto

//go:build !protoopaque

package ownerv1

import (
	_ "buf.build/gen/go/bufbuild/protovalidate/protocolbuffers/go/buf/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type GetOwnersRequest struct {
	state protoimpl.MessageState `protogen:"hybrid.v1"`
	// The Users or Organizations to request.
	OwnerRefs     []*OwnerRef `protobuf:"bytes,1,rep,name=owner_refs,json=ownerRefs,proto3" json:"owner_refs,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetOwnersRequest) Reset() {
	*x = GetOwnersRequest{}
	mi := &file_buf_registry_owner_v1_owner_service_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetOwnersRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetOwnersRequest) ProtoMessage() {}

func (x *GetOwnersRequest) ProtoReflect() protoreflect.Message {
	mi := &file_buf_registry_owner_v1_owner_service_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

func (x *GetOwnersRequest) GetOwnerRefs() []*OwnerRef {
	if x != nil {
		return x.OwnerRefs
	}
	return nil
}

func (x *GetOwnersRequest) SetOwnerRefs(v []*OwnerRef) {
	x.OwnerRefs = v
}

type GetOwnersRequest_builder struct {
	_ [0]func() // Prevents comparability and use of unkeyed literals for the builder.

	// The Users or Organizations to request.
	OwnerRefs []*OwnerRef
}

func (b0 GetOwnersRequest_builder) Build() *GetOwnersRequest {
	m0 := &GetOwnersRequest{}
	b, x := &b0, m0
	_, _ = b, x
	x.OwnerRefs = b.OwnerRefs
	return m0
}

type GetOwnersResponse struct {
	state protoimpl.MessageState `protogen:"hybrid.v1"`
	// The retrieved Users or Organizations in the same order as requested.
	Owners        []*Owner `protobuf:"bytes,1,rep,name=owners,proto3" json:"owners,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetOwnersResponse) Reset() {
	*x = GetOwnersResponse{}
	mi := &file_buf_registry_owner_v1_owner_service_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetOwnersResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetOwnersResponse) ProtoMessage() {}

func (x *GetOwnersResponse) ProtoReflect() protoreflect.Message {
	mi := &file_buf_registry_owner_v1_owner_service_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

func (x *GetOwnersResponse) GetOwners() []*Owner {
	if x != nil {
		return x.Owners
	}
	return nil
}

func (x *GetOwnersResponse) SetOwners(v []*Owner) {
	x.Owners = v
}

type GetOwnersResponse_builder struct {
	_ [0]func() // Prevents comparability and use of unkeyed literals for the builder.

	// The retrieved Users or Organizations in the same order as requested.
	Owners []*Owner
}

func (b0 GetOwnersResponse_builder) Build() *GetOwnersResponse {
	m0 := &GetOwnersResponse{}
	b, x := &b0, m0
	_, _ = b, x
	x.Owners = b.Owners
	return m0
}

var File_buf_registry_owner_v1_owner_service_proto protoreflect.FileDescriptor

const file_buf_registry_owner_v1_owner_service_proto_rawDesc = "" +
	"\n" +
	")buf/registry/owner/v1/owner_service.proto\x12\x15buf.registry.owner.v1\x1a!buf/registry/owner/v1/owner.proto\x1a\x1bbuf/validate/validate.proto\"_\n" +
	"\x10GetOwnersRequest\x12K\n" +
	"\n" +
	"owner_refs\x18\x01 \x03(\v2\x1f.buf.registry.owner.v1.OwnerRefB\v\xbaH\b\x92\x01\x05\b\x01\x10\xfa\x01R\townerRefs\"S\n" +
	"\x11GetOwnersResponse\x12>\n" +
	"\x06owners\x18\x01 \x03(\v2\x1c.buf.registry.owner.v1.OwnerB\b\xbaH\x05\x92\x01\x02\b\x01R\x06owners2s\n" +
	"\fOwnerService\x12c\n" +
	"\tGetOwners\x12'.buf.registry.owner.v1.GetOwnersRequest\x1a(.buf.registry.owner.v1.GetOwnersResponse\"\x03\x90\x02\x01BUZSbuf.build/gen/go/bufbuild/registry/protocolbuffers/go/buf/registry/owner/v1;ownerv1b\x06proto3"

var file_buf_registry_owner_v1_owner_service_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_buf_registry_owner_v1_owner_service_proto_goTypes = []any{
	(*GetOwnersRequest)(nil),  // 0: buf.registry.owner.v1.GetOwnersRequest
	(*GetOwnersResponse)(nil), // 1: buf.registry.owner.v1.GetOwnersResponse
	(*OwnerRef)(nil),          // 2: buf.registry.owner.v1.OwnerRef
	(*Owner)(nil),             // 3: buf.registry.owner.v1.Owner
}
var file_buf_registry_owner_v1_owner_service_proto_depIdxs = []int32{
	2, // 0: buf.registry.owner.v1.GetOwnersRequest.owner_refs:type_name -> buf.registry.owner.v1.OwnerRef
	3, // 1: buf.registry.owner.v1.GetOwnersResponse.owners:type_name -> buf.registry.owner.v1.Owner
	0, // 2: buf.registry.owner.v1.OwnerService.GetOwners:input_type -> buf.registry.owner.v1.GetOwnersRequest
	1, // 3: buf.registry.owner.v1.OwnerService.GetOwners:output_type -> buf.registry.owner.v1.GetOwnersResponse
	3, // [3:4] is the sub-list for method output_type
	2, // [2:3] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_buf_registry_owner_v1_owner_service_proto_init() }
func file_buf_registry_owner_v1_owner_service_proto_init() {
	if File_buf_registry_owner_v1_owner_service_proto != nil {
		return
	}
	file_buf_registry_owner_v1_owner_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_buf_registry_owner_v1_owner_service_proto_rawDesc), len(file_buf_registry_owner_v1_owner_service_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_buf_registry_owner_v1_owner_service_proto_goTypes,
		DependencyIndexes: file_buf_registry_owner_v1_owner_service_proto_depIdxs,
		MessageInfos:      file_buf_registry_owner_v1_owner_service_proto_msgTypes,
	}.Build()
	File_buf_registry_owner_v1_owner_service_proto = out.File
	file_buf_registry_owner_v1_owner_service_proto_goTypes = nil
	file_buf_registry_owner_v1_owner_service_proto_depIdxs = nil
}
