// Code generated by "stringer -type=Type"; DO NOT EDIT.

package attribute

import "strconv"

func _() {
	// An "invalid array index" compiler error signifies that the constant values have changed.
	// Re-run the stringer command to generate them again.
	var x [1]struct{}
	_ = x[EMPTY-0]
	_ = x[BOOL-1]
	_ = x[INT64-2]
	_ = x[FLOAT64-3]
	_ = x[STRING-4]
	_ = x[BOOLSLICE-5]
	_ = x[INT64SLICE-6]
	_ = x[FLOAT64SLICE-7]
	_ = x[STRINGSLICE-8]
	_ = x[BYTESLICE-9]
	_ = x[SLICE-10]
}

const _Type_name = "EMPTYBOOLINT64FLOAT64STRINGBOOLSLICEINT64SLICEFLOAT64SLICESTRINGSLICEBYTESLICESLICE"

var _Type_index = [...]uint8{0, 5, 9, 14, 21, 27, 36, 46, 58, 69, 78, 83}

func (i Type) String() string {
	idx := int(i) - 0
	if i < 0 || idx >= len(_Type_index)-1 {
		return "Type(" + strconv.FormatInt(int64(i), 10) + ")"
	}
	return _Type_name[_Type_index[idx]:_Type_index[idx+1]]
}
